@@ -6,16 +6,13 @@
 import torch
 import torch.nn as nn
 
-from typing import Dict, List, Optional, Tuple, Union
+from typing import List, Tuple, Union
 
 from PIL import Image
 import torchvision
 import torchvision.transforms as T
 
-from lightly.transforms import GaussianBlur
-from lightly.transforms import Jigsaw
-from lightly.transforms import RandomRotate
-from lightly.transforms import RandomSolarization
+from lightly.transforms import GaussianBlur, Jigsaw, RandomRotate, RandomSolarization
 
 imagenet_normalize = {
     'mean': [0.485, 0.456, 0.406],
@@ -125,7 +122,13 @@ class ImageCollateFunction(BaseCollateFunction):
         hf_prob:
             Probability that horizontal flip is applied.
         rr_prob:
-            Probability that random (+90 degree) rotation is applied.
+            Probability that random rotation is applied.
+        rr_degrees:
+            Range of degrees to select from for random rotation. If rr_degrees is None, 
+            images are rotated by 90 degrees. If rr_degrees is a (min, max) tuple, 
+            images are rotated by a random angle in [min, max]. If rr_degrees is a
+            single number, images are rotated by a random angle in 
+            [-rr_degrees, +rr_degrees]. All rotations are counter-clockwise.
         normalize:
             Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
 
@@ -145,6 +148,7 @@ class ImageCollateFunction(BaseCollateFunction):
                  vf_prob: float = 0.0,
                  hf_prob: float = 0.5,
                  rr_prob: float = 0.0,
+                 rr_degrees: Union[None, float, Tuple[float, float]] = None,
                  normalize: dict = imagenet_normalize):
 
         if isinstance(input_size, tuple):
@@ -158,7 +162,7 @@ class ImageCollateFunction(BaseCollateFunction):
 
         transform = [T.RandomResizedCrop(size=input_size,
                                          scale=(min_scale, 1.0)),
-             RandomRotate(prob=rr_prob),
+             _random_rotation_transform(rr_prob=rr_prob, rr_degrees=rr_degrees),
              T.RandomHorizontalFlip(p=hf_prob),
              T.RandomVerticalFlip(p=vf_prob),
              T.RandomApply([color_jitter], p=cj_prob),
@@ -216,6 +220,7 @@ class MultiViewCollateFunction(nn.Module):
         fnames = [fname for _, _, fname in batch]
         return views, labels, fnames
 
+
 class SimCLRCollateFunction(ImageCollateFunction):
     """Implements the transformations for SimCLR.
 
@@ -239,7 +244,13 @@ class SimCLRCollateFunction(ImageCollateFunction):
         hf_prob:
             Probability that horizontal flip is applied.
         rr_prob:
-            Probability that random (+90 degree) rotation is applied.
+            Probability that random rotation is applied.
+        rr_degrees:
+            Range of degrees to select from for random rotation. If rr_degrees is None, 
+            images are rotated by 90 degrees. If rr_degrees is a (min, max) tuple, 
+            images are rotated by a random angle in [min, max]. If rr_degrees is a
+            single number, images are rotated by a random angle in 
+            [-rr_degrees, +rr_degrees]. All rotations are counter-clockwise.
         normalize:
             Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
 
@@ -267,6 +278,7 @@ class SimCLRCollateFunction(ImageCollateFunction):
                  vf_prob: float = 0.0,
                  hf_prob: float = 0.5,
                  rr_prob: float = 0.0,
+                 rr_degrees: Union[None, float, Tuple[float, float]] = None,
                  normalize: dict = imagenet_normalize):
 
         super(SimCLRCollateFunction, self).__init__(
@@ -283,6 +295,7 @@ class SimCLRCollateFunction(ImageCollateFunction):
             vf_prob=vf_prob,
             hf_prob=hf_prob,
             rr_prob=rr_prob,
+            rr_degrees=rr_degrees,
             normalize=normalize,
         )
 
@@ -312,7 +325,13 @@ class MoCoCollateFunction(ImageCollateFunction):
         hf_prob:
             Probability that horizontal flip is applied.
         rr_prob:
-            Probability that random (+90 degree) rotation is applied.
+            Probability that random rotation is applied.
+        rr_degrees:
+            Range of degrees to select from for random rotation. If rr_degrees is None, 
+            images are rotated by 90 degrees. If rr_degrees is a (min, max) tuple, 
+            images are rotated by a random angle in [min, max]. If rr_degrees is a
+            single number, images are rotated by a random angle in 
+            [-rr_degrees, +rr_degrees]. All rotations are counter-clockwise.
         normalize:
             Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
 
@@ -339,6 +358,7 @@ class MoCoCollateFunction(ImageCollateFunction):
                  vf_prob: float = 0.0,
                  hf_prob: float = 0.5,
                  rr_prob: float = 0.0,
+                 rr_degrees: Union[None, float, Tuple[float, float]] = None,
                  normalize: dict = imagenet_normalize):
 
         super(MoCoCollateFunction, self).__init__(
@@ -355,10 +375,13 @@ class MoCoCollateFunction(ImageCollateFunction):
             vf_prob=vf_prob,
             hf_prob=hf_prob,
             rr_prob=rr_prob,
+            rr_degrees=rr_degrees,
             normalize=normalize,
         )
 
 
+
+
 class MultiCropCollateFunction(MultiViewCollateFunction):
     """Implements the multi-crop transformations for SwaV.
 
@@ -434,7 +457,13 @@ class SwaVCollateFunction(MultiCropCollateFunction):
         vf_prob:
             Probability that vertical flip is applied.
         rr_prob:
-            Probability that random (+90 degree) rotation is applied.
+            Probability that random rotation is applied.
+        rr_degrees:
+            Range of degrees to select from for random rotation. If rr_degrees is None, 
+            images are rotated by 90 degrees. If rr_degrees is a (min, max) tuple, 
+            images are rotated by a random angle in [min, max]. If rr_degrees is a
+            single number, images are rotated by a random angle in 
+            [-rr_degrees, +rr_degrees]. All rotations are counter-clockwise.
         cj_prob:
             Probability that color jitter is applied.
         cj_strength:
@@ -469,6 +498,7 @@ class SwaVCollateFunction(MultiCropCollateFunction):
                  hf_prob: float = 0.5,
                  vf_prob: float = 0.0,
                  rr_prob: float = 0.0,
+                 rr_degrees: Union[None, float, Tuple[float, float]] = None,
                  cj_prob: float = 0.8,
                  cj_strength: float = 0.8,
                  random_gray_scale: float = 0.2,
@@ -483,7 +513,7 @@ class SwaVCollateFunction(MultiCropCollateFunction):
         transforms = T.Compose([
             T.RandomHorizontalFlip(p=hf_prob),
             T.RandomVerticalFlip(p=vf_prob),
-            RandomRotate(prob=rr_prob),
+            _random_rotation_transform(rr_prob=rr_prob, rr_degrees=rr_degrees),
             T.ColorJitter(),
             T.RandomApply([color_jitter], p=cj_prob),
             T.RandomGrayscale(p=random_gray_scale),
@@ -526,7 +556,13 @@ class DINOCollateFunction(MultiViewCollateFunction):
         vf_prob:
             Probability that vertical flip is applied.
         rr_prob:
-            Probability that random (+90 degree) rotation is applied.
+            Probability that random rotation is applied.
+        rr_degrees:
+            Range of degrees to select from for random rotation. If rr_degrees is None, 
+            images are rotated by 90 degrees. If rr_degrees is a (min, max) tuple, 
+            images are rotated by a random angle in [min, max]. If rr_degrees is a
+            single number, images are rotated by a random angle in 
+            [-rr_degrees, +rr_degrees]. All rotations are counter-clockwise.
         cj_prob:
             Probability that color jitter is applied.
         cj_bright:
@@ -564,6 +600,7 @@ class DINOCollateFunction(MultiViewCollateFunction):
         hf_prob=0.5,
         vf_prob=0,
         rr_prob=0,
+        rr_degrees: Union[None, float, Tuple[float, float]] = None,
         cj_prob=0.8,
         cj_bright=0.4,
         cj_contrast=0.4,
@@ -580,7 +617,7 @@ class DINOCollateFunction(MultiViewCollateFunction):
         flip_and_color_jitter = T.Compose([
             T.RandomHorizontalFlip(p=hf_prob),
             T.RandomVerticalFlip(p=vf_prob),
-            RandomRotate(prob=rr_prob),
+            _random_rotation_transform(rr_prob=rr_prob, rr_degrees=rr_degrees),
             T.RandomApply(
                 [T.ColorJitter(
                     brightness=cj_bright, 
@@ -754,7 +791,7 @@ class PIRLCollateFunction(nn.Module):
         color_jitter = T.ColorJitter(
             cj_bright, cj_contrast, cj_sat, cj_hue
         )
-        
+
         # Transform for transformed jigsaw image
         transform = [
             T.RandomHorizontalFlip(p=hf_prob),
@@ -894,3 +931,92 @@ class MSNCollateFunction(MultiViewCollateFunction):
         transforms = [transform] * random_views
         transforms += [focal_transform] * focal_views
         super().__init__(transforms=transforms)
+
+
+class SMoGCollateFunction(MultiViewCollateFunction):
+    """Implements the transformations for SMoG.
+
+    Attributes:
+        crop_sizes:
+            Size of the input image in pixels for each crop category.
+        crop_counts:
+            Number of crops for each crop category.
+        crop_min_scales:
+            Min scales for each crop category.
+        crop_max_scales:
+            Max_scales for each crop category.
+        gaussian_blur_probs:
+            Probability of Gaussian blur for each crop category.
+        gaussian_blur_kernel_sizes:
+            Kernel size of Gaussian blur for each crop category.
+        solarize_probs:
+            Probability of solarization for each crop category.
+        hf_prob:
+            Probability that horizontal flip is applied.
+        cj_prob:
+            Probability that color jitter is applied.
+        cj_strength:
+            Strength of the color jitter.
+        random_gray_scale:
+            Probability of conversion to grayscale.
+        normalize:
+            Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
+
+    """
+
+    def __init__(
+        self,
+        crop_sizes: List[int] = [224, 96],
+        crop_counts: List[int] = [4, 4],
+        crop_min_scales: List[float] = [0.2, 0.05],
+        crop_max_scales: List[float] = [1.0, 0.2],
+        gaussian_blur_probs: List[float] = [0.5, 0.1],
+        gaussian_blur_kernel_sizes: List[float] = [0.1, 0.1],
+        solarize_probs: List[float] = [0.0, 0.2],
+        hf_prob: float = 0.5,
+        cj_prob: float = 1.0,
+        cj_strength: float = 0.5,
+        random_gray_scale: float = 0.2,
+        normalize: dict = imagenet_normalize,
+    ):
+
+        transforms = []
+        for i in range(len(crop_sizes)):
+
+            random_resized_crop = T.RandomResizedCrop(
+                crop_sizes[i],
+                scale=(crop_min_scales[i], crop_max_scales[i])
+            )
+
+            color_jitter = T.ColorJitter(
+                0.8 * cj_strength,
+                0.8 * cj_strength,
+                0.4 * cj_strength,
+                0.2 * cj_strength,
+            )
+
+            transforms.extend([
+                T.Compose([
+                    random_resized_crop,
+                    T.RandomHorizontalFlip(p=hf_prob),
+                    T.RandomApply([color_jitter], p=cj_prob),
+                    T.RandomGrayscale(p=random_gray_scale),
+                    GaussianBlur(prob=gaussian_blur_probs[i], kernel_size=gaussian_blur_kernel_sizes[i]), # TODO
+                    RandomSolarization(prob=solarize_probs[i]),
+                    T.ToTensor(),
+                    T.Normalize(mean=normalize["mean"], std=normalize["std"]),
+                ])
+            ] * crop_counts[i])
+
+        super().__init__(transforms)
+
+def _random_rotation_transform(
+    rr_prob: float,
+    rr_degrees: Union[None, float, Tuple[float, float]],
+) -> Union[RandomRotate, T.RandomApply]:
+    if rr_degrees is None:
+        # Random rotation by 90 degrees.
+        return RandomRotate(prob=rr_prob, angle=90)
+    else:
+        # Random rotation with random angle defined by rr_degrees.
+        return T.RandomApply([T.RandomRotation(degrees=rr_degrees)], p=rr_prob)
